@@ -7,7 +7,7 @@ package Lemonldap::NG::Portal::AuthMulti;
 
 use Lemonldap::NG::Portal::_Multi;    #inherits
 
-our $VERSION = '1.9.1';
+our $VERSION = '1.9.3';
 
 sub authInit {
     my $self = shift;
@@ -43,7 +43,9 @@ sub authLogout {
 
 sub authForce {
     my $self = shift;
-    return $self->_multi->try( 'authForce', 0 );
+    $self->_sub('authInit');
+    my $s    = $self->{_multi}->{last}->[0] . "::authForce";
+    return &{$s}($self);
 }
 
 sub getDisplayType {
