@@ -71,7 +71,7 @@ use Digest::MD5;
 #inherits Apache::Session
 #link Lemonldap::NG::Common::Apache::Session::SOAP protected globalStorage
 
-our $VERSION = '1.9.3';
+our $VERSION = '1.9.4';
 
 use base qw(Lemonldap::NG::Common::CGI Exporter);
 our @ISA;
@@ -907,6 +907,13 @@ sub redirect {
 sub getApacheSession {
     my ( $self, $id, $noInfo, $force ) = @_;
 
+    if ($id) {
+        $self->lmLog( "Try to get session $id", 'debug' );
+    }
+    else {
+        $self->lmLog( "Try to get a new session", 'debug' );
+    }
+
     my $apacheSession = Lemonldap::NG::Common::Session->new(
         {
             storageModule        => $self->{globalStorage},
@@ -920,7 +927,7 @@ sub getApacheSession {
     );
 
     if ( $apacheSession->error ) {
-        $self->lmLog( $apacheSession->error, 'debug' );
+        $self->lmLog( $apacheSession->error, 'error' );
         return;
     }
 
@@ -929,11 +936,30 @@ sub getApacheSession {
         return;
     }
 
+    my $now = time;
+    if (
+        $id
+        and (
+            $now - $apacheSession->data->{_utime} > $self->{timeout}
+            or (    $self->{timeoutActivity}
+                and $apacheSession->data->{_lastSeen}
+                and $now - $apacheSession->data->{_lastSeen} >
+                $self->{timeoutActivity} )
+        )
+      )
+    {
+        $self->lmLog( "Session $id expired", 'debug' );
+        return;
+    }
+
     unless ($noInfo) {
         $self->setApacheUser( $apacheSession->data->{ $self->{whatToTrace} } )
           if ($id);
         $self->{id} = $apacheSession->id;
     }
+
+    $self->lmLog( "Return session " . $apacheSession->id, 'debug' );
+
     return $apacheSession;
 }
 
@@ -1863,7 +1889,8 @@ sub controlExistingSession {
             # A session has been found => call existingSession
             my $r = $self->_sub( 'existingSession', $id, $self->{sessionInfo} );
             if ( $r == PE_DONE ) {
-                $self->{user}  = $self->{sessionInfo}->{_whatToTrace};
+                $self->{user} = $self->{sessionInfo}
+                  ->{ $self->{whatToTrace} || '_whatToTrace' };
                 $self->{error} = $self->_subProcess(
                     qw(checkNotification issuerDBInit authInit issuerForAuthUser authFinish autoRedirect)
                 );
